/*
  CapacitiveSense.h v.02 - Capacitive Sensing Library for 'duino / Wiring
  Copyright (c) 2008 Paul Bagder  All right reserved.
*/


// include this library's description file
#include "CapacitiveSense.h"

// include core Wiring API
#include "WProgram.h"


// include description files for other libraries used (if any)
#include "pins_arduino.h"
#include "WConstants.h"




// Constructor /////////////////////////////////////////////////////////////////
// Function that handles the creation and setup of instances

int32_t capSenseRaw(uint8_t sendPin, uint8_t receivePin, uint8_t samples){     // sendPin (send pin) sends voltage to receivePin (receiver pin) through high value resistor
  //  body capacitance makes up RC time constant, sensed by receivePin

  long total = 0;
  int i;

  pinMode(sendPin, OUTPUT);                         // do this to take care of PWM timers that might be on
  digitalWrite(sendPin,LOW);


  // get pin mapping and port for send Pin - from PinMode function in core

  uint8_t sBit =  digitalPinToBitMask(sendPin);   // get send pin's ports and bitmask
  uint8_t sPort = digitalPinToPort(sendPin);
  volatile uint8_t *sReg;
  volatile uint8_t *sOut;
  sOut = portOutputRegister(sPort);
  *sOut &= ~sBit;

  uint8_t rBit = digitalPinToBitMask(receivePin);    // get receive pin's ports and bitmask 
  uint8_t rPort = digitalPinToPort(receivePin);
  volatile uint8_t *rReg;
  volatile uint8_t *rIn;
  volatile uint8_t *rOut;

  rOut = portOutputRegister(rPort);

  if (sPort == NOT_A_PIN) return -1;
  sReg = portModeRegister(sPort);

  // get pin mapping and port for receive Pin - from digital pin functions in Wiring.c


  if (rPort == NOT_A_PIN) return -1;
  rReg = portModeRegister(rPort);
  rOut = portOutputRegister(rPort);
  rIn  = portInputRegister(rPort);

  *rReg &= ~rBit;                         // set receive Pin to input
  *rOut &= ~rBit;                         // set Output Register low to make sure pullups are off

    total = 0; 
    for(i = 0; i < samples; i++){             // multple samples to take out jitter

      // set receive pin LOW briefly to discharge fully - because the last while loop  will have exited when pin is around ~ 2.5V 

    *rReg |= rBit;         // set pin to OUTPUT - pin is now LOW AND OUTPUT
    *rReg &= ~rBit;         // set pin to INPUT 

    *sOut |= sBit;     // set send Pin High

    while( !(*rIn & rBit)  && (total < CAPSENSETIMEOUT)){        // while receive pin is LOW AND total is less than timeout
      total++;
    }

    // set receive pin HIGH briefly to charge up fully - because the while loop above will exit when pin is ~ 2.5V 
    *rOut  |= rBit;        // set receive pin HIGH - turns on pullup 
    *rReg |= rBit;         // set pin to OUTPUT - pin is now HIGH AND OUTPUT
    *rReg &= ~rBit;         // set pin to INPUT 
    *rOut  &= ~rBit;       // turn off pullup

    *sOut &= ~sBit;     // set send Pin LOW

    while( (*rIn & rBit) && (total < CAPSENSETIMEOUT)){        // while receive pin is HIGH  AND total is less than timeout
      total++;
    }

  } 
    
  if (total < CAPSENSETIMEOUT){
    return total / samples; 
  }
  else{         // function timed out
    return -1;
  }
}

//**********************************

int32_t capSense(uint8_t sendPin, uint8_t receivePin, uint8_t samples){       // autocalibrate to baseline 0 reading

int32_t total = capSenseRaw(sendPin, receivePin, samples);


if (gCount % CAPSENSERESET == 0) gLeastTotal = 0x0FFFFFFFL;   // reset for "autocalibrate"
if (total < gLeastTotal) gLeastTotal = total;
gCount++;
return(total - gLeastTotal);

}




