

/**
 * Arduino light sensor game
 * by Roman A. Anastasini
 */
import processing.serial.*;
import cc.arduino.*;

int windowWidth = 600;
int windowHeight = 400;
int x = 0;
boolean collision = false;
int durchm = 20;
int startTime = 0;
int endTime = 0;
PFont fontA;
int arduino = 200;
boolean start = false;
float foo;
float bar;
void setup() {
  size(windowWidth, windowHeight);        //  window size
  background(0);                          //  background color
  createScene();                          //  call function to create the scene
  fontA = loadFont("Ziggurat-HTF-Black-32.vlw");    //  set font (must be locatet in the data folder)
  textFont(fontA, 16);                              //  Set the font and its size (in units of pixels)
  mouseY = 150;                           //  set mouse position
  arduino = mouseY;
}

void draw() {                            //  called directly after setup, runs continously until noLoop() is called
  arduino = mouseY;
  if(keyPressed) {
    if (key == 's') {
      start = true;
      startTime = millis();                   //  set start time = actual timestamp in mili seconds
    }
  }
  if (start) {
    if (!collision) { 
      createScene();
      stroke(0);                           //  set pen color
      drawCircle(x, arduino);               //  call the function to draw the little ball
      drawCircle2(x, (int)(((float)x/(float)windowWidth) * 100.0)+100);
      text((int)(((float)x/(float)windowWidth) * 100.0)+100, 100, 200);
      collision = detectCollision(x, arduino);      //  check for collision (top, buttom line or end of window
       x++;                                //  increase the balls x-position
    } else {
      endGame();                           //  cal the function which will end the game and write your score
    }
  } else {
    createScene();
    drawCircle(0, arduino);
  }
}

void createScene() {                    //  function which will draw the scene (top and buttom line, etc)
  
  stroke(204, 102, 0);
  fill(204, 102, 0);
  //  upper field
  beginShape();
    vertex(0, 0);
    vertex(0, 100);
    vertex(windowWidth,300);
    vertex(windowWidth,0);
  endShape(CLOSE);
  
  // buttom field
  stroke(204, 102, 0);
  fill(204, 102, 0);
  beginShape();
    vertex(0, windowHeight);
    vertex(0, 200);
    vertex(windowWidth, 400);
    vertex(windowWidth, windowHeight);
  endShape(CLOSE);
  
  //  middel field
  fill(125);
  stroke(0);
  beginShape();
    vertex(0, 100);
    vertex(windowWidth, 300);
    vertex(windowWidth, 400);
    vertex(0, 200);
  endShape();
}

void drawCircle(int x, int y) {                  // draws our little ball
  fill(255);
  ellipse(x, y, durchm, durchm);
  noFill();
}

void drawCircle2(int x, int y) {                  // draws our little ball
  fill(255);
  ellipse(x, y, 3, 3);
  noFill();
}

boolean detectCollision(int x, int y){           //  check whether the ball touches one of the evil lines
  boolean result = false;
  int value = 0;
  if (x >= windowWidth-durchm/2) {               //  check for end of window
    result = true;
  }
  // start check for collision with top line
  if (y < ((int)(((float)x/(float)windowWidth) * 200.0)+100)) {
    result = true;
  }
  if (y > ((float)(((float)x/(float)windowWidth) * 200.0) + 200))
  {
    result = true;
  }
  return result;
}

void endGame(){
  endTime = millis();
  noLoop();
  fill(0);
  text("your score: ", 50, 60);
  text(endTime-startTime, 200, 60);
  // maybe we could place a restart button here...  :) 
}

